#include "DataProcessor32.hpp"

DataProcessor32::DataProcessor32(HistoData &h, const sc_tdc_format &f)
: iDataProcessor(h), data_len_(0), data_offset_(0),
channel_len_(0), channel_offset_(0)
{
	set_data_len(f.time_data_length);
	set_data_offset(f.time_data_offset);
	set_channel_len(f.channel_length);
	set_channel_offset(f.channel_offset);
}

int DataProcessor32::process(void *vbuf, int binning)
{
	int channel;
	int data;
	int maxY = std::numeric_limits <unsigned> ::min();
	unsigned int *buf = (unsigned int *)vbuf;
	for (int i=0; i<histo_data_.total_events; ++i) {
		channel = extract_channel(buf[i]);
		histo_data_.rates[channel] += 1.;
		if (channel != 0) {
			histo_data_.rates[channel-1] = histo_data_.rates[channel-1];
		}
		if (channel != CHANNELS-1) {
			histo_data_.rates[channel+1] = histo_data_.rates[channel+1];
		}
		data = extract_data(buf[i], binning);
		if (data < histo_data_.histosize) {
			if (minX_ > data) {
				minX_ = data;
			}
			if (maxX_ < data) {
				maxX_ = data;
			}
			histo_data_.field[channel][data] += 1.0;
			if (data != 0) {
				histo_data_.field[channel][data-1] = histo_data_.field[channel][data-1];
			}
			if (data != histo_data_.histosize) {
				histo_data_.field[channel][data+1] = histo_data_.field[channel][data+1];
			}
			if (maxY < histo_data_.field[channel][data]) {
				maxY = int(histo_data_.field[channel][data]);
			}
		}
	}
	return maxY;
}

int DataProcessor32::extract_channel(unsigned int sample) const
{
	if (channel_len_ < 1) {
		return 0;
	}

	return (sample >> channel_offset_) & sc_mask32[channel_len_];
}

int DataProcessor32::extract_data(unsigned int sample, int binning) const
{
	if (data_len_ < 1) {
		return 0;
	}
	int ret = (sample >> data_offset_) & sc_mask32[data_len_];
	ret >>= binning;
	return ret;
}

void DataProcessor32::set_data_len(unsigned char l)
{
	data_len_ = l;
}

void DataProcessor32::set_data_offset(unsigned char o)
{
	data_offset_ = o;
}

void DataProcessor32::set_channel_len(unsigned char l)
{
	channel_len_ = l;
}

void DataProcessor32::set_channel_offset(unsigned char o)
{
	channel_offset_ = o;
}
