#include "HistoData.hpp"
#include <QApplication>
#include <fstream>
#include <iomanip>
#include <scTDC.h>
#include <QStringList>

HistoData::HistoData()
: histosize(0), total_events(0)
{
	reset();
}

void HistoData::reset()
{
	histosize = 0;
	total_events = 0;
	rates.clear();
	for (int i=0; i<CHANNELS; ++i) {
		rates.append(0.);
		field.clear();
		field.insert(i, QMap <double, double>());
	}
}
bool HistoData::save(const QString filename_full)
{
	if (histosize == 0) {
		return false;
	}
	
	QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
	QString filename = QFileInfo(filename_full).absolutePath();
	filename += "/";
	filename += QFileInfo(filename_full).baseName();
	std::ofstream out;
	QString filename_rates = filename + "_rates.txt";
	out.open(filename_rates.toStdString().c_str(), std::ios::out | std::ios::trunc);
	out << "Rates (X:channel_number, Y:countrate):" << std::endl;
	out << "         X\tY" << std::endl;
	for (int i=0; i<CHANNELS; ++i) {
		out << std::setw(10) << i << "\t" << rates[i] << std::endl;
	}
	out.close();

	for (int j=0; j<CHANNELS; ++j) {
		if (rates[j] == 0) {
			continue;
		}
		QString filename_channel(filename);
		filename_channel.append("_ch");
		filename_channel.append(QString::number(j));
		filename_channel.append(".txt");

		out.open(filename_channel.toStdString().c_str(), std::ios::out | std::ios::trunc);
		QList <double> keys = field[j].keys();
		out << "Channel" << j << " (Time[ns] vs Countrate):" << std::endl;
		out << std::setprecision(12);
		for (int i=0; i<keys.size(); ++i) {
			out << bin_to_ns(keys.at(i)) << "\t" << field[j][keys.at(i)] << std::endl;
		}
		out.close();
	}

	QApplication::restoreOverrideCursor();
	return true;
}

double bin_to_ns(long long b)
{
	double k;
	sc_tdc_get_binsize(&k);
	return b *k;
}

long long ns_to_bin(double n)
{
	double k;
	sc_tdc_get_binsize(&k);
	return n /k;
}
