#include "Mwidget.h"
#include <QCoreApplication>
#include <QRegExp>
#include <QMessageBox>
#include <QFile>
#include <QSettings>
#include <QAction>
#include <scTDC.h>
#include <QDebug>
#include <cstdlib>

#define ERROR_CHECK(x) if((x) < 0){ \
							do_error_check(ret); \
							return; \
						}

Mwidget::Mwidget(QWidget *parent)
: QWidget(parent), buffer(0), state(IDLE),
histosize(10000LL)
{
	init_objects();
	init_layout();
	init_connections();
	setLayout(layout);
}

void Mwidget::init_objects(void)
{
	spinbox_buffersize = new QSpinBox;
	spinbox_mestime = new QSpinBox;
	spinbox_iterations_todo = new QSpinBox;
	spinbox_iters_left = new QSpinBox;
	spinbox_iters_left->setEnabled(false);
	spinbox_event_threshold = new QSpinBox;
	spinbox_binsize = new QDoubleSpinBox;
	spinbox_binsize->setEnabled(false);
	spinbox_histosize = new QSpinBox;
	button_bufalloc = new QPushButton("buffer alloc");
	button_bufalloc->setEnabled(false);
	button_start = new QPushButton("start");
	button_start->setEnabled(false);
	histo = new Mhisto;
	rates = new Mrates;
	label_autorun = new QLabel("autorun:");
	label_buffer_size = new QLabel("buffer size:");
	label_mes_time = new QLabel("mes. time [ms]:");
	label_accumulate = new QLabel("accumulate:");
	label_iterations_todo = new QLabel("iterations:");
	label_iters_left = new QLabel("iterations left:");
	label_event_threshold = new QLabel("event threshold:");
	label_binsize = new QLabel("binsize [ps]:");
	label_histosize = new QLabel("histosize [ns]:");
	autorun = new QCheckBox;
	checkbox_accumulate = new QCheckBox;
	layout = new QHBoxLayout;
	flayout = new QFormLayout;
	spinbox_buffersize->setMaximum(1000000000);
	spinbox_mestime->setMaximum(10000000);
	spinbox_buffersize->setValue(1000000);
	spinbox_mestime->setValue(1000);
	spinbox_event_threshold->setRange(0, 1000000000);
	spinbox_iterations_todo->setRange(1, 1000000000);
	spinbox_iters_left->setRange(0, 1000000000);
	spinbox_histosize->setRange(0, 30000000);
	spinbox_histosize->setValue(10000);
}

void Mwidget::init_connections(void)
{
	connect(
		button_bufalloc,
		SIGNAL(clicked()),
		this,
		SLOT(buffer_alloc())
	);
	connect(
		button_start,
		SIGNAL(clicked()),
		this,
		SLOT(start_stop())
	);
	connect(
		this,
		SIGNAL(takeBuffer(unsigned int *, int, int, long long)),
		histo,
		SLOT(newBuffer(unsigned int*, int, int, long long))
	);
	connect(
		histo,
		SIGNAL(takeRates(QList <double>)),
		rates,
		SLOT(newRates(QList <double>))
	);
	connect(
		checkbox_accumulate,
		SIGNAL(stateChanged(int)),
		histo,
		SLOT(set_accumulation(int))
	);
	connect(
		this,
		SIGNAL(do_event_reaction(int)),
		this,
		SLOT(event_reaction(int)),
		Qt::QueuedConnection
	);
	connect(
		spinbox_histosize,
		SIGNAL(valueChanged(int)),
		this,
		SLOT(histosize_changed(int))
	);
}

void Mwidget::init_layout(void)
{
	flayout->addRow(label_buffer_size, spinbox_buffersize);
	flayout->addRow(button_bufalloc);
	flayout->addRow(label_mes_time, spinbox_mestime);
	flayout->addRow(label_autorun, autorun);
	flayout->addRow(label_accumulate, checkbox_accumulate);
	flayout->addRow(label_iterations_todo, spinbox_iterations_todo);
	flayout->addRow(label_event_threshold, spinbox_event_threshold);
	flayout->addRow(button_start);
	flayout->addRow(label_iters_left, spinbox_iters_left);
	flayout->addRow(label_binsize, spinbox_binsize);
	flayout->addRow(label_histosize, spinbox_histosize);
	flayout->addRow(rates);
	layout->addWidget(histo);
	layout->addLayout(flayout);
	layout->setStretchFactor(histo, 2);
}

void Mwidget::init_deinit(bool deinited)
{
	if (deinited) {
		int ret;
		ret = sc_tdc_init("./tdc_gpx3.ini");
		if (0 > ret) {
			do_error_check(ret);
			QAction *a = reinterpret_cast <QAction *> (sender());
			a->setChecked(false);
			return;
		}
		ERROR_CHECK(ret);
		ret = sc_tdc_set_complete_callback(
				  this,
				  (void (*)(void *, int))callback
			  );
		ERROR_CHECK(ret);
		buffer_alloc();
		button_bufalloc->setEnabled(true);
		double k;
		sc_tdc_get_binsize(&k);
		spinbox_binsize->setValue(k*1000);
		ret = histo->init_data_processor();
		if (0 > ret) {
			do_error_check(ret);
			QAction *a = reinterpret_cast <QAction *> (sender());
			a->setChecked(false);
			return;
		}
		emit show_this("Device is initialized");
	} else {
		int ret;
		if (state == RUNNING) {
			autorun->setCheckState(Qt::Unchecked);
			start_stop();
		}
		while (state != IDLE) {
			QCoreApplication::processEvents();
		}
		ret = sc_tdc_deinit();
		ERROR_CHECK(ret);
		button_bufalloc->setEnabled(false);
		button_start->setEnabled(false);
		emit show_this("Device is deinitialized");
	}
}

void Mwidget::buffer_alloc(void)
{
	int ret;
	ret = sc_tdc_alloc_buffer(spinbox_buffersize->value(), &buffer);
	if (ret < 0) {
		button_start->setEnabled(false);
	} else {
		button_start->setEnabled(true);
	}
	ERROR_CHECK(ret);
}

void Mwidget::start_stop(void)
{
	int ret;
	switch (state) {
		case IDLE:
		case STOPPING:
			spinbox_iters_left->setValue(spinbox_iterations_todo->value());
			histo->erase_data_store();
			show_this("exposure running");
			ret = sc_tdc_start_measure(spinbox_mestime->value());
			ERROR_CHECK(ret);
			button_start->setText("stop");
			state = RUNNING;
			break;
		case RUNNING:
			ret = sc_tdc_interrupt();
			ERROR_CHECK(ret);
			button_start->setText("start");
			autorun->setChecked(false);
			state = STOPPING;
			break;
	}
}

void Mwidget::callback(void *th_s, int a)
{
	(static_cast <Mwidget *> (th_s))->event_reaction_signalizer(a);
}

void Mwidget::event_reaction_signalizer(int a)
{
	emit do_event_reaction(a);
}

void Mwidget::event_reaction(int err)
{
	int ret;
	int events;
	char errstr[ERRSTRLEN];
	bool rerun = false;
	sc_get_err_msg(err, errstr);
	emit show_this(QString(errstr));
	ret = sc_tdc_get_events_count(&events);
	ERROR_CHECK(ret);
	emit takeBuffer(
		buffer,
		events,
		0,
		histosize
	);

	bool accu = checkbox_accumulate->isChecked();
	bool arun = autorun->isChecked();

	if (accu && arun) {
		int v = spinbox_iters_left->value();
		--v;
		if (v > 0) {
			spinbox_iters_left->setValue(v);
		} else {
			spinbox_iters_left->setValue(spinbox_iterations_todo->value());
			histo->erase_data_store();
		}
		rerun = true;
	} else if (accu && !arun) {
		int v = spinbox_iters_left->value();
		--v;
		spinbox_iters_left->setValue(v);
		if (v > 0) {
			rerun = true;
		} else {
			rerun = false;
		}
	} else if (!accu && arun) {
		rerun = true;
	} else if (!accu && !arun) {
		rerun = false;
	}

	int threshold = spinbox_event_threshold->value();
	int total = histo->histo_data().total_events;
	if (threshold != 0 && total > threshold) {
		rerun = false;
	}

	if (rerun) {
		ret = sc_tdc_start_measure(spinbox_mestime->value());
		ERROR_CHECK(ret);
	} else {
		state = IDLE;
		button_start->setText("start");
	}
}

void Mwidget::do_error_check(int err)
{
	char errstr[ERRSTRLEN];
	sc_get_err_msg(err, errstr);
	QMessageBox::critical(this, "Error", QString(errstr));
}

void Mwidget::save_data(void)
{
	if (state != IDLE) {
		return;
	}

	if (!histo->save_data()) {
		QMessageBox::critical(this, "Error", "Could not save data");
	}
}

void Mwidget::histosize_changed(int h)
{
	if (h == 0) {
		histosize = 30000000LL;
	} else {
		histosize = h;
	}
}
